@@ -7,7 +7,7 @@ use base qw( Rose::Object );
 use Net::LDAP::Class::MethodMaker (
     scalar => [qw( base_dn ldap object_classes )], );
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 =head1 NAME
 
@@ -133,6 +133,20 @@ sub interrogate {
 
 }
 
+=head2 base_dn
+
+Get/set the base DN used by interrogate().
+
+=head2 ldap
+
+Get/set the Net::LDAP object.
+
+=head2 object_classes
+
+Get/set the array ref of object classes to be used by interrogate().
+
+=cut
+
 1;
 
 __END__
