@@ -8,7 +8,7 @@ use Net::LDAP::Class::MethodMaker (
     'related_objects'       => [qw( group groups )],
 );
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 =head1 NAME
 
@@ -131,14 +131,14 @@ sub stringify { shift->username }
 
 =head2 username
 
-Returns the value of the first unique attribute.
+Get/set the value of the first unique attribute.
 
 =cut
 
 sub username {
     my $self = shift;
     my $attr = $self->unique_attributes->[0];
-    return $self->$attr;
+    return $self->$attr(@_);
 }
 
 =head2 random_string([I<len>])
@@ -153,6 +153,9 @@ my @charset = (
     ',',        '$',        '?',        '@',        '!'
 );
 
+# sanity check for collisions in tight loops
+my %rand_string_cache;
+
 sub random_string {
     my $self = shift;
     my $len = shift || 10;
@@ -173,7 +176,8 @@ sub random_string {
     until (    $str =~ /\d/
             && $str =~ /[A-Z]/
             && $str =~ /[a-z]/
-            && $str =~ /\W/ )
+            && $str =~ /\W/
+            && !$rand_string_cache{$str}++ )
     {
         @chars = ();
         for ( my $i = 0; $i <= ( $len - 1 ); $i++ ) {
