@@ -9,7 +9,7 @@ use Net::LDAP::Class::MethodMaker (
     'scalar --get_set_init' => [qw( default_home_dir default_email_suffix )],
 );
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 =head1 NAME
 
@@ -139,14 +139,14 @@ sub fetch_group {
 
     # because AD does not store primaryGroupToken but computes it,
     # we must do gymnastics using SIDs
-    #warn "gid = $gid";
+    warn "gid = $gid";
 
     my $user_sid_string = _sid2string( $self->objectSID );
 
-    #warn "user_sid_string:  $user_sid_string";
+    warn "user_sid_string:  $user_sid_string";
     ( my $group_sid_string = $user_sid_string ) =~ s/\-[^\-]+$/-$gid/;
 
-    #warn "group_sid_string: $group_sid_string";
+    warn "group_sid_string: $group_sid_string";
 
     return $class->new(
         objectSID => $group_sid_string,
@@ -156,9 +156,12 @@ sub fetch_group {
 
 sub _sid2string {
     my $sid = shift;
+    carp "nlc sid    = " . Data::Dump::dump($sid);
     my (@unpack) = unpack( "H2 H2 n N V*", $sid );
     my ( $sid_rev, $num_auths, $id1, $id2, @ids ) = (@unpack);
-    return join( "-", "S", $sid_rev, ( $id1 << 32 ) + $id2, @ids );
+    my $string = join( "-", "S", $sid_rev, ( $id1 << 32 ) + $id2, @ids );
+    carp "nlc string = $string";
+    return $string;
 }
 
 sub _string2sid {
@@ -180,6 +183,9 @@ sub _string2sid {
         $sid .= pack( "I", $i );
     }
 
+    carp "nlc string = $string";
+    carp "nlc sid    = " . Data::Dump::dump($sid);
+
     return $sid;
 }
 
@@ -369,10 +375,10 @@ sub action_for_create {
         givenName      => $givenName,
         displayName    => $cn,
         sn             => $sn,
-        cn             => $cn,
-        homeDirectory  => $self->default_home_dir . "\\$username",
-        mail           => $email,
-        userAccountControl => 512,     # so AD treats it as a Normal user
+        cn             => $cn,          # must match $dn below
+        homeDirectory => $self->default_home_dir . "\\$username",
+        mail          => $email,
+        userAccountControl => 512,      # so AD treats it as a Normal user
         unicodePwd         => $pass,
     );
 
@@ -381,8 +387,10 @@ sub action_for_create {
     # mix in whatever has been set
     for my $name ( keys %{ $self->{_not_yet_set} } ) {
 
+        next if $name eq 'cn';    # because we alter this in setup_for_write()
+
         #warn "set $name => $self->{_not_yet_set}->{$name}";
-        unless ( exists $attr{$name} ) {
+        if ( !exists $attr{$name} ) {
             $attr{$name} = delete $self->{_not_yet_set}->{$name};
         }
         else {
@@ -469,7 +477,7 @@ sub setup_for_write {
         else {
             my $group_obj = $self->fetch_group($group);
             if ( !$group_obj ) {
-                croak "no such group in AD server: $group";
+                confess "no such group in AD server: $group";
             }
             $gid = $group_obj->gid;
         }
@@ -494,8 +502,13 @@ sub setup_for_write {
     my $cn = $self->cn;
     $cn = join( ' ', $givenName, $sn ) unless defined $cn;
 
+    my $un = $self->username;
+    if ( $cn ne $un and $cn !~ m!/$un$! ) {
+        $cn .= "/$un";    # for uniqueness
+    }
+
     my $email = $self->mail;
-    $email = ( $self->username . $self->default_email_suffix )
+    $email = ( $un . $self->default_email_suffix )
         unless defined $email;
 
     return ( $group, $gid, $givenName, $sn, $cn, $email );
@@ -539,6 +552,8 @@ sub action_for_update {
     my %replace;
     for my $attr ( keys %{ $self->{_was_set} } ) {
 
+        next if $attr eq 'cn';    # because we mangle in setup_for_write()
+
         my $old = $self->{_was_set}->{$attr}->{old};
         my $new = $self->{_was_set}->{$attr}->{new} || $derived{$attr};
 
